﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Linq.Expressions;
using System.Drawing.Imaging;
using System.Drawing;

namespace Esmf
{
    public class OutputHelper
    {
        public static void ShowModel(ModelFields modelFields)
        {
            var app = new System.Windows.Application();

            var w = new ModelFieldsWindow(modelFields);

            app.Run(w);
        }

        public static void WriteParameterToFile<D1, T>(IEnumerable<Parameter1DimensionalMember<D1, T>> parameter, TextWriter file)
        {
            string listSeperator = System.Globalization.CultureInfo.CurrentUICulture.TextInfo.ListSeparator;
            foreach (var x in parameter)
            {
                string attachedDimensions = x.AttachedDimensions.Aggregate("", (workingString, next) => workingString + next + listSeperator);
                file.WriteLine("{0}{1}{2}{3}", attachedDimensions, x.Dimension1, listSeperator, x.Value);
            }
            file.Flush();
        }

        public static void WriteParameterToFile<D1, T>(IEnumerable<Parameter1DimensionalMember<D1, T>> parameter, string filename)
        {
            using (var f = File.CreateText(filename))
            {
                OutputHelper.WriteParameterToFile(parameter, f);
            }
        }

        public static void WriteParameterToFile<D1, T>(IParameter1Dimensional<D1, T> parameter, TextWriter file)
        {
            OutputHelper.WriteParameterToFile(parameter.GetEnumerator(), file);
        }

        public static void WriteParameterToFile<D1, T>(IVariable1Dimensional<D1, T> parameter, TextWriter file)
        {
            OutputHelper.WriteParameterToFile(parameter as IParameter1Dimensional<D1, T>, file);
        }

        public static void WriteParameterToFile<D1, T>(IParameter1Dimensional<D1, T> parameter, string filename)
        {
            using (var f = File.CreateText(filename))
            {
                OutputHelper.WriteParameterToFile(parameter, f);
            }
        }

        public static void WriteParameterToFile<D1, T>(IVariable1Dimensional<D1, T> parameter, string filename)
        {
            OutputHelper.WriteParameterToFile(parameter as IParameter1Dimensional<D1, T>, filename);
        }

        public static void WriteParameterToFile<D1, D2, T>(IEnumerable<Parameter2DimensionalMember<D1, D2, T>> parameter, TextWriter file)
        {
            string listSeperator = System.Globalization.CultureInfo.CurrentUICulture.TextInfo.ListSeparator;
            foreach (var x in parameter)
            {
                string attachedDimensions = x.AttachedDimensions.Aggregate("", (workingString, next) => workingString + next + listSeperator);
                file.WriteLine("{0}{1}{2}{3}{4}{5}", attachedDimensions, x.Dimension1, listSeperator, x.Dimension2, listSeperator, x.Value);
            }
            file.Flush();
        }

        public static void WriteParametersToFile<D1, D2, T>(string filename, params IEnumerable<Parameter2DimensionalMember<D1, D2, T>>[] parameters)
        {
            using (var f = File.CreateText(filename))
            {
                OutputHelper.WriteParametersToFile(f, parameters);
            }

        }

        public static void WriteParametersToFile<D1, D2, T>(TextWriter file, params IEnumerable<Parameter2DimensionalMember<D1, D2, T>>[] parameters)
        {
                        string listSeperator = System.Globalization.CultureInfo.CurrentUICulture.TextInfo.ListSeparator;

            var keys = new HashSet<Tuple<string, D1, D2>>();

            var parametersAsDict = new List<Dictionary<Tuple<string,D1, D2>, Parameter2DimensionalMember<D1, D2, T>>>();

            foreach (var p in parameters)
            {
                var lkeys = from i in p
                            select Tuple.Create(i.AttachedDimensions.Aggregate("", (workingString, next) => workingString + next + listSeperator), i.Dimension1, i.Dimension2);
                foreach (var key in lkeys)
                    keys.Add(key);

                var currentDict = new Dictionary<Tuple<string,D1, D2>, Parameter2DimensionalMember<D1, D2, T>>();
                parametersAsDict.Add(currentDict);
                foreach (var x in p)
                    currentDict.Add(Tuple.Create(x.AttachedDimensions.Aggregate("", (workingString, next) => workingString + next + listSeperator), x.Dimension1, x.Dimension2), x);
            }
            Console.WriteLine("We have {0} keys with {1} parameters", keys.Count, parameters.Length);

            foreach (var key in keys)
            {

                file.Write("{0}{1}{2}{3}", key.Item1, key.Item2, listSeperator, key.Item3);
                foreach (var dict in parametersAsDict)
                {
                    Parameter2DimensionalMember<D1, D2, T> val;
                    if (dict.TryGetValue(key, out val))
                    {
                        file.Write("{0}{1}", listSeperator, val.Value);
                    }
                }
                file.WriteLine();
            }
            file.Flush();

        }

        public static void WriteParametersToFile<D1, D2, T>(string filename, params IParameter2Dimensional<D1, D2, T>[] parameters)
        {            
            var d = parameters.Select(i=>i.GetEnumerator()).ToArray();
            OutputHelper.WriteParametersToFile<D1,D2,T>(filename, d);
        }

        public static void WriteParametersToFile<D1, D2, T>(string filename, params IVariable2Dimensional<D1, D2, T>[] parameters)
        {
            var d = parameters.Select(i => i.GetEnumerator()).ToArray();
            OutputHelper.WriteParametersToFile<D1, D2, T>(filename, d);
        }


        public static void WriteParameterToFile<D1, D2, T>(IEnumerable<Parameter2DimensionalMember<D1, D2, T>> parameter, string filename)
        {
            using (var f = File.CreateText(filename))
            {
                OutputHelper.WriteParameterToFile(parameter, f);
            }
        }

        public static void WriteParameterToFile<D1, D2, T>(IParameter2Dimensional<D1, D2, T> parameter, TextWriter file)
        {
            OutputHelper.WriteParameterToFile(parameter.GetEnumerator(), file);
        }

        public static void WriteParameterToFile<D1, D2, T>(IParameter2Dimensional<D1, D2, T> parameter, string filename)
        {
            using (var f = File.CreateText(filename))
            {
                OutputHelper.WriteParameterToFile(parameter, f);
            }
        }

        public static void WriteParameterToFile<D1, D2, T>(IVariable2Dimensional<D1, D2, T> parameter, string filename)
        {
            OutputHelper.WriteParameterToFile(parameter as IParameter2Dimensional<D1, D2, T>, filename);
        }

        public static void WriteParameterToFile<D1, D2, T>(IVariable2Dimensional<D1, D2, T> parameter, TextWriter file)
        {
            OutputHelper.WriteParameterToFile(parameter as IParameter2Dimensional<D1, D2, T>, file);
        }

        public static void PlotParameterToFile<D1, T>(IParameter1Dimensional<D1, T> parameter, string filename)
        {
            var plot = new NPlot.LinePlot();
            plot.AbscissaData = (from d in parameter.GetEnumerator()
                                 select d.Dimension1).ToArray();
            plot.DataSource = (from d in parameter.GetEnumerator()
                               select d.Value).ToArray();
            plot.Label = "asdf";

            var surface = new NPlot.Bitmap.PlotSurface2D(975, 625);
            surface.Clear();
            //surface.Title = "Some title should go here";
            surface.TitleFont = new System.Drawing.Font("Arial", 12);
            surface.BackColor = Color.White;
            surface.Add(new NPlot.Grid(), NPlot.PlotSurface2D.XAxisPosition.Bottom, NPlot.PlotSurface2D.YAxisPosition.Left);
            surface.Add(plot, NPlot.PlotSurface2D.XAxisPosition.Bottom, NPlot.PlotSurface2D.YAxisPosition.Left);

            //surface.XAxis1.Label = "year";
            //surface.XAxis1.LabelFont = new System.Drawing.Font("Arial", 12);
            //surface.XAxis1.TickTextFont = new System.Drawing.Font("Arial", 12);
            ////surface.XAxis1.NumberFormat = "f2";
            //surface.XAxis1.TicksLabelAngle = 90;
            //surface.XAxis1.TickTextNextToAxis = true;
            //surface.XAxis1.FlipTicksLabel = true;
            //surface.XAxis1.LabelOffset = 110;
            //surface.XAxis1.LabelOffsetAbsolute = true;

            //surface.YAxis1.Label = "year";
            //surface.YAxis1.LabelFont = new System.Drawing.Font("Arial", 12);
            //surface.YAxis1.TickTextFont = new System.Drawing.Font("Arial", 12);
            ////surface.YAxis1.NumberFormat = "f2";
            //surface.YAxis1.TicksLabelAngle = 90;
            //surface.YAxis1.TickTextNextToAxis = true;
            //surface.YAxis1.FlipTicksLabel = true;
            //surface.YAxis1.LabelOffset = 110;
            //surface.YAxis1.LabelOffsetAbsolute = true;

            //var legend = new NPlot.Legend();
            //legend.AttachTo(NPlot.PlotSurface2D.XAxisPosition.Top, NPlot.PlotSurface2D.YAxisPosition.Right);
            //legend.VerticalEdgePlacement = NPlot.Legend.Placement.Inside;
            //legend.HorizontalEdgePlacement = NPlot.Legend.Placement.Outside;
            //legend.BorderStyle = NPlot.LegendBase.BorderType.Line;
            //surface.Legend = legend;

            surface.Refresh();

            surface.Bitmap.Save(filename, ImageFormat.Png);
        }

        public static void PlotParameterToFile<D1, T>(IVariable1Dimensional<D1, T> parameter, string filename)
        {
            OutputHelper.PlotParameterToFile(parameter as IParameter1Dimensional<D1, T>, filename);
        }

        public static void PlotParameterToFile<D1, D2, T>(IParameter2Dimensional<D1, D2, T> parameter, string filename)
        {
            if (typeof(D1) != typeof(Timestep) || !typeof(D2).IsEnum)
            {
                throw new ArgumentException();
            }

            var colors = new Color[] 
                { 
                    Color.Red, 
                    Color.Blue,
                    Color.Green,
                    Color.Yellow,
                    Color.Brown,
                    Color.Coral,
                    Color.DarkBlue,
                    Color.Beige,
                    Color.Cyan,
                    Color.HotPink,
                    Color.Orchid,
                    Color.SlateGray,
                    Color.SpringGreen,
                    Color.Thistle,
                    Color.LawnGreen,
                    Color.Khaki
                };

            var plots = from d in parameter.GetEnumerator()
                        group d by d.Dimension2 into p
                        select new NPlot.LinePlot()
                        {
                            Label = p.Key.ToString(),
                            AbscissaData = (from a in p select a.Dimension1).ToArray(),
                            DataSource = (from a in p select a.Value).ToArray()
                        };


            var surface = new NPlot.Bitmap.PlotSurface2D(975, 625);
            surface.Clear();
            //surface.Title = "Some title should go here";
            surface.TitleFont = new System.Drawing.Font("Arial", 12);
            surface.BackColor = Color.White;
            surface.Add(new NPlot.Grid(), NPlot.PlotSurface2D.XAxisPosition.Bottom, NPlot.PlotSurface2D.YAxisPosition.Left);

            var cenumerator = colors.GetEnumerator();

            foreach (var plot in plots)
            {
                cenumerator.MoveNext();
                plot.Color = (Color)cenumerator.Current;
                surface.Add(plot, NPlot.PlotSurface2D.XAxisPosition.Bottom, NPlot.PlotSurface2D.YAxisPosition.Left);
            }
            surface.XAxis1.Label = "year";
            //surface.XAxis1.LabelFont = new System.Drawing.Font("Arial", 12);
            //surface.XAxis1.TickTextFont = new System.Drawing.Font("Arial", 12);
            //surface.XAxis1.NumberFormat = "{0:F}";
            //surface.XAxis1.TicksLabelAngle = 90;
            //surface.XAxis1.TickTextNextToAxis = true;
            //surface.XAxis1.FlipTicksLabel = true;
            //surface.XAxis1.LabelOffset = 110;
            //surface.XAxis1.LabelOffsetAbsolute = true;

            //surface.YAxis1.Label = "year";
            //surface.YAxis1.LabelFont = new System.Drawing.Font("Arial", 12);
            //surface.YAxis1.TickTextFont = new System.Drawing.Font("Arial", 12);
            surface.YAxis1.NumberFormat = "{0:N}";
            //surface.YAxis1.TicksLabelAngle = 90;
            //surface.YAxis1.TickTextNextToAxis = true;
            //surface.YAxis1.FlipTicksLabel = true;
            //surface.YAxis1.LabelOffset = 110;
            //surface.YAxis1.LabelOffsetAbsolute = true;

            var legend = new NPlot.Legend();
            legend.AttachTo(NPlot.PlotSurface2D.XAxisPosition.Top, NPlot.PlotSurface2D.YAxisPosition.Left);
            legend.VerticalEdgePlacement = NPlot.Legend.Placement.Inside;
            legend.HorizontalEdgePlacement = NPlot.Legend.Placement.Inside;
            legend.BorderStyle = NPlot.LegendBase.BorderType.Line;
            legend.BackgroundColor = Color.White;
            surface.Legend = legend;

            surface.Refresh();

            surface.Bitmap.Save(filename, ImageFormat.Png);
        }

        public static void PlotParameterToFile<D1, D2, T>(IVariable2Dimensional<D1, D2, T> parameter, string filename)
        {
            OutputHelper.PlotParameterToFile(parameter as IParameter2Dimensional<D1, D2, T>, filename);
        }

    }
}
