﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using Esmf;
using System;
using Fund.CommonDimensions;

namespace Fund
{

    public class FundWorkflow
    {
        private Esmf.ModelFields _modelFields = new Esmf.ModelFields();
        //private ParameterValues _parameters;

        public Fund.Components.SocioEconomic.ISocioEconomicState SocioEconomicState;
        public Fund.Components.Emissions.IEmissionsState EmissionsState;
        public Fund.Components.Welfare.IGlobalWelfareState GlobalWelfareState;
        public Fund.Components.Welfare.IRegionalWelfareState RegionalWelfareState;
        public Fund.Components.ImpactAggregation.IImpactAggregationState ImpactAggregationState;
        public Fund.Components.Climate.IClimateState ClimateState;
        public Fund.Components.ClimateRegional.IClimateRegionalState ClimateRegionalState;
        public Fund.Components.Ocean.IOceanState OceanState;
        public Fund.Components.BioDiversity.IBioDiversityState BioDiversityState;
        public Fund.Components.ImpactBioDiversity.IImpactBioDiversityState ImpactBioDiversityState;
        public Fund.IFundOld2Climate FundOld2ClimateState;
        public Fund.ISocioEconomic2Welfare FundOld2GlobalWelfareState;
        public Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState ScenarioUncertaintyState;
        public Fund.Components.ImpactHeating.IImpactHeatingState ImpactHeatingState;
        public Fund.Components.ImpactCooling.IImpactCoolingState ImpactCoolingState;
        public Fund.Components.ImpactAgriculture.IImpactAgricultureState ImpactAgricultureState;
        public Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState GlobalHotellingTaxationState;
        public Fund.Components.ImpactWaterResources.IImpactWaterResourcesState ImpactWaterResourcesState;
        public Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState ImpactDiarrhoeaState;
        public Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState ImpactTropicalStormsState;
        public Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState ImpactExtratropicalStormsState;
        public Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState ImpactSeaLevelRiseState;
        public Fund.Components.Geography.IGeographyState GeographyState;
        public Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState TerrestrialBiosphereState;
        public Fund.Components.ImpactForests.IImpactForestsState ImpactForestsState;
        public Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState ImpactVectorBorneDiseases;
        public Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState ImpactCardiovascularRespiratoryState;
        public Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState ImpactDeathMorbidityState;

        public Esmf.ModelFields ModelFields { get { return _modelFields; } }

        public void Run(ModelOutput output=null)
        {
            ReCreateStateVariables();

            var clock = new Clock(Timestep.FromSimulationYear(0), Timestep.FromSimulationYear(LegacyConstants.NYear));

            RunEmissionsComponent(clock);
            RunScenarioUncertaintyComponent(clock);
            RunGeographyComponent(clock);
            RunImpactSeaLevelRiseComponent(clock);
            RunTerrestrialBiosphereComponent(clock);
            RunSocioEconomicComponent(clock);

            clock.Advance();

            while (!clock.IsDone)
            {
                RunScenarioUncertaintyComponent(clock);

                RunGeographyComponent(clock);

                RunSocioEconomicComponent(clock);

                RunGlobalHotellingTaxationComponent(clock);

                RunEmissionsComponent(clock);

                RunTerrestrialBiosphereComponent(clock);

                RunFundOld2ClimateState(clock);
                RunClimateComponent(clock);
                RunClimateRegionalComponent(clock);

                RunOceanComponent(clock);

                RunBioDiversityComponent(clock);

                RunImpactBioDiversityComponent(clock);

                RunImpactHeatingComponent(clock);
                RunImpactCoolingComponent(clock);

                RunImpactAgricultureComponent(clock);

                RunImpactWaterResourcesComponent(clock);

                RunImpactDiarrhoeaComponent(clock);

                RunImpactTropicalStormsComponent(clock);

                RunImpactExtratropicalStormsComponent(clock);

                RunImpactSeaLevelRiseComponent(clock);

                RunImpactForestsComponent(clock);

                RunImpactVectorBorneDiseasesComponent(clock);

                RunImpactCardiovascularRespiratoryComponent(clock);

                RunImpactDeathMorbidityComponent(clock);

                foreach (var r in Dimension<Region>.Values)
                {

                    if (output != null)
                    {
                        var t = clock.Current;

                        output.Damages[t.Value, (int)r, Sector.sWater] = -ImpactWaterResourcesState.water[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sForests] = -ImpactForestsState.forests[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sHeating] = -ImpactHeatingState.heating[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sCooling] = -ImpactCoolingState.cooling[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sAgriculture] = -ImpactAgricultureState.agcost[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sDryland] = ImpactSeaLevelRiseState.drycost[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sSeaProtection] = ImpactSeaLevelRiseState.protcost[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sImigration] = ImpactSeaLevelRiseState.entercost[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sHurrican] = ImpactTropicalStormsState.hurrdam[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sExtratropicalStorms] = ImpactExtratropicalStormsState.extratropicalstormsdam[clock.Current, r] * 1000000000;

                        output.Damages[t.Value, (int)r, Sector.sSpecies] = ImpactBioDiversityState.species[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sDeath] = ImpactDeathMorbidityState.deadcost[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sMorbidity] = ImpactDeathMorbidityState.morbcost[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sWetland] = ImpactSeaLevelRiseState.wetcost[clock.Current, r] * 1000000000;
                        output.Damages[t.Value, (int)r, Sector.sEmigration] = ImpactSeaLevelRiseState.leavecost[clock.Current, r] * 1000000000;

                        // Add GDP for that year and region to the Output object
                        // income is multiplied by 1 billion, since that is the unit
                        // of income
                        output.Incomes.Add(clock.Current.Value, (int)r, SocioEconomicState.income[clock.Current, r] * 1000000000);

                        // Add population for that year and region to the Output object
                        // population is multiplied by 1 million, since that is the unit of
                        // population
                        output.Populations.Add(clock.Current.Value, (int)r, SocioEconomicState.population[clock.Current, r] * 1000000);
                    }
                }

                RunImpactAggregationComponent(clock);

                RunFundOld2GlobalWelfareState(clock);
                RunGlobalWelfareComponent(clock);

                RunRegionalWelfareComponent(clock);

                clock.Advance();
            }
        }

        public void RunSocioEconomicComponent(Clock clock)
        {
            // Set up SocioEconomic component
            var lSocioEconomicComponent = new Fund.Components.SocioEconomic.SocioEconomicComponent();

            // Run SocioEconomic component
            lSocioEconomicComponent.Run(clock, SocioEconomicState);
        }

        public void RunClimateComponent(Clock clock)
        {
            var t = clock.Current;

            // Set up climate component
            var lclimateComp = new Fund.Components.Climate.ClimateComponent();

            // Run climate component
            lclimateComp.Run(clock, ClimateState);

        }

        public void InitClimateState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "ClimateSensitivity", parameters);
            _modelFields.AddNonDimensionalVariable<double>("climate", "so2ind", 0.08);
            _modelFields.AddNonDimensionalVariable<double>("climate", "so2dir", 0.03);
            _modelFields.AddNonDimensionalVariable<double>("climate", "sf6pre", 0.04);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "lifesf6", parameters);
            _modelFields.AddNonDimensionalVariable<double>("climate", "n2opre", 285);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "lifen2o", parameters);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2pre", 275);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2frac5", 0.10);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2frac4", 0.25);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2frac3", 0.32);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2frac2", 0.20);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2frac1", 0.13);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "lifeco1", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "lifeco2", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "lifeco3", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "lifeco4", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "lifeco5", parameters);
            _modelFields.AddNonDimensionalVariable<double>("climate", "ch4pre", 790);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "lifech4", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Esmf.Timestep, double>("climate", "RunAwayCH4", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "ch4add", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "n2oadd", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "sf6add", parameters);
            _modelFields.AddNonDimensionalVariable<bool>("climate", "addmp", false);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "LifeTempConst", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "LifeTempLin", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("climate", "LifeTempQd", parameters);

            // Parameters coming from different components, need to be coupled
            //_modelFields.ConnectParameterToVariableNonDimensional<double>("climate","ClimateSensitivity","roebakerclimatesensitivity", "ClimateSensitivity");
            _modelFields.ConnectParameterToVariable("climate", "mco2", "emissions", "mco2");
            _modelFields.ConnectParameterToVariable("climate", "globso2", "emissions", "globso2");
            _modelFields.ConnectParameterToVariable("climate", "globsf6", "emissions", "globsf6");
            _modelFields.ConnectParameterToVariable("climate", "globn2o", "emissions", "globn2o");
            _modelFields.ConnectParameterToVariable("climate", "globch4", "emissions", "globch4");
            _modelFields.ConnectParameterToVariable("climate", "addco2", "fundold2climate", "addco2");
            _modelFields.ConnectParameterToVariable("climate", "addch4", "fundold2climate", "addch4");
            _modelFields.ConnectParameterToVariable("climate", "addn2o", "fundold2climate", "addn2o");
            _modelFields.ConnectParameterToVariable("climate", "addsf6", "fundold2climate", "addsf6");
            _modelFields.ConnectParameterToVariable("climate", "TerrestrialCO2", "terrestrialbiosphere", "TerrestrialCO2");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Climate.IClimateState));
            ClimateState = ss.ConnectToState<Fund.Components.Climate.IClimateState>(_modelFields, "climate");

            // Initial values, should eventually also come from parameter file
            ClimateState.temp[Timestep.FromSimulationYear(0)] = 0.20;
            ClimateState.cbox1[Timestep.FromSimulationYear(0)] = 283.53;
            ClimateState.cbox2[Timestep.FromSimulationYear(0)] = 5.62;
            ClimateState.cbox3[Timestep.FromSimulationYear(0)] = 6.29;
            ClimateState.cbox4[Timestep.FromSimulationYear(0)] = 2.19;
            ClimateState.cbox5[Timestep.FromSimulationYear(0)] = 0.15;
            ClimateState.acco2[Timestep.FromSimulationYear(0)] = ClimateState.cbox1[Timestep.FromSimulationYear(0)]
                + ClimateState.cbox2[Timestep.FromSimulationYear(0)]
                + ClimateState.cbox3[Timestep.FromSimulationYear(0)]
                + ClimateState.cbox4[Timestep.FromSimulationYear(0)]
                + ClimateState.cbox5[Timestep.FromSimulationYear(0)];
            ClimateState.acsf6[Timestep.FromSimulationYear(0)] = ClimateState.sf6pre;
            ClimateState.acn2o[Timestep.FromSimulationYear(0)] = 296;
            ClimateState.acch4[Timestep.FromSimulationYear(0)] = 1222;
            ClimateState.maxrf = 0;
        }

        public void InitSocioEconomicState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("socioeconomic", "consleak", parameters);
            _modelFields.AddNonDimensionalVariable<bool>("socioeconomic", "runwithoutdamage", false);
            _modelFields.AddNonDimensionalVariable<bool>("socioeconomic", "runwithoutpopulationperturbation", false);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("socioeconomic", "plusel", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("socioeconomic", "gdp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("socioeconomic", "pop90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("socioeconomic", "gdp0", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("socioeconomic", "pop0", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("socioeconomic", "plus90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("socioeconomic", "urbcorr", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("socioeconomic", "ypcgrowth", "scenariouncertainty", "ypcgrowth");
            _modelFields.ConnectParameterToVariable("socioeconomic", "pgrowth", "scenariouncertainty", "pgrowth");

            // Parameters coming from different components, need to be coupled
            _modelFields.ConnectParameterToVariable("socioeconomic", "leave", "impactsealevelrise", "leave");
            _modelFields.ConnectParameterToVariable("socioeconomic", "enter", "impactsealevelrise", "enter");
            _modelFields.ConnectParameterToVariable("socioeconomic", "eloss", "impactaggregation", "eloss");
            _modelFields.ConnectParameterToVariable("socioeconomic", "sloss", "impactaggregation", "sloss");
            _modelFields.ConnectParameterToVariable("socioeconomic", "dead", "impactdeathmorbidity", "dead");
            _modelFields.ConnectParameterToVariable("socioeconomic", "mitigationcost", "emissions", "mitigationcost");
            _modelFields.ConnectParameterToVariable("socioeconomic", "area", "geography", "area");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.SocioEconomic.ISocioEconomicState));
            SocioEconomicState = ss.ConnectToState<Fund.Components.SocioEconomic.ISocioEconomicState>(_modelFields, "socioeconomic");
        }

        public FundWorkflow(ParameterValues parameters)
        {
            InitVariables(typeof(Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState), "scenariouncertainty");
            InitVariables(typeof(Fund.Components.BioDiversity.IBioDiversityState), "biodiversity");
            InitVariables(typeof(Fund.Components.Climate.IClimateState), "climate");
            InitVariables(typeof(Fund.Components.ClimateRegional.IClimateRegionalState), "climateregional");
            InitVariables(typeof(Fund.Components.ImpactBioDiversity.IImpactBioDiversityState), "impactbiodiversity");
            InitVariables(typeof(Fund.Components.ImpactHeating.IImpactHeatingState), "impactheating");
            InitVariables(typeof(Fund.Components.ImpactAgriculture.IImpactAgricultureState), "impactagriculture");
            InitVariables(typeof(Fund.Components.ImpactCooling.IImpactCoolingState), "impactcooling");
            InitVariables(typeof(Fund.Components.Ocean.IOceanState), "ocean");
            InitVariables(typeof(Fund.Components.SocioEconomic.ISocioEconomicState), "socioeconomic");
            InitVariables(typeof(Fund.Components.Emissions.IEmissionsState), "emissions");
            InitVariables(typeof(Fund.Components.Welfare.IGlobalWelfareState), "globalwelfare");
            InitVariables(typeof(Fund.Components.Welfare.IRegionalWelfareState), "regionalwelfare");
            InitVariables(typeof(Fund.IFundOld2Climate), "fundold2climate");
            InitVariables(typeof(Fund.ISocioEconomic2Welfare), "fundold2globalwelfare");
            InitVariables(typeof(Fund.Components.ImpactAggregation.IImpactAggregationState), "impactaggregation");
            InitVariables(typeof(Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState), "globalhotellingtaxation");
            InitVariables(typeof(Fund.Components.ImpactWaterResources.IImpactWaterResourcesState), "impactwaterresources");
            InitVariables(typeof(Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState), "impactdiarrhoea");
            InitVariables(typeof(Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState), "impacttropicalstorms");
            InitVariables(typeof(Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState), "impactextratropicalstorms");
            InitVariables(typeof(Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState), "impactsealevelrise");
            InitVariables(typeof(Fund.Components.Geography.IGeographyState), "geography");
            InitVariables(typeof(Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState), "terrestrialbiosphere");
            InitVariables(typeof(Fund.Components.ImpactForests.IImpactForestsState), "impactforests");
            InitVariables(typeof(Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState), "impactvectorbornediseases");
            InitVariables(typeof(Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState), "impactcardiovascularrespiratory");
            InitVariables(typeof(Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState), "impactdeathmorbidity");

            InitScenarioUncertaintyState(parameters);
            InitClimateState(parameters);
            InitClimateRegionalState(parameters);
            InitOceanState(parameters);
            InitBioDiversityState(parameters);
            InitSocioEconomicState(parameters);
            InitEmissionsState(parameters);
            InitGlobalWelfareState(parameters);
            InitRegionalWelfareState(parameters);
            InitImpactBioDiversityState(parameters);
            InitImpactHeatingState(parameters);
            InitImpactAgricultureState(parameters);
            InitImpactCoolingState(parameters);
            InitFundOld2ClimateState(parameters);
            InitFundOld2GlobalWelfare(parameters);
            InitImpactAggregationState(parameters);
            InitGlobalHotellingTaxationState(parameters);
            InitImpactWaterResourcesState(parameters);
            InitImpactDiarrhoeaState(parameters);
            InitImpactTropicalStormsState(parameters);
            InitImpactExtratropicalStormsState(parameters);
            InitImpactSeaLevelRiseState(parameters);
            InitGeographyState(parameters);
            InitTerrestrialBiosphereState(parameters);
            InitImpactForestsState(parameters);
            InitImpactVectorBorneDiseasesState(parameters);
            InitImpactCardiovascularRespiratoryState(parameters);
            InitImpactDeathMorbidityState(parameters);
        }

        public void InitOceanState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("ocean", "lifesea", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("ocean", "seas", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("ocean", "temp", "climate", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Ocean.IOceanState));
            OceanState = ss.ConnectToState<Fund.Components.Ocean.IOceanState>(_modelFields, "ocean");

            // Initial values, should eventually also come from parameter file
            var t0 = Timestep.FromSimulationYear(0);
            OceanState.sea[t0] = 0;
        }

        public void RunOceanComponent(Clock clock)
        {
            var t = clock.Current;

            // Set up ocean component
            var oc = new Fund.Components.Ocean.OceanComponent();

            // Run ocean component
            oc.Run(clock, OceanState);
        }

        public void InitBioDiversityState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("biodiversity", "nospecbase", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("biodiversity", "bioloss", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("biodiversity", "biosens", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("biodiversity", "dbsta", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("biodiversity", "temp", "climate", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.BioDiversity.IBioDiversityState));
            BioDiversityState = ss.ConnectToState<Fund.Components.BioDiversity.IBioDiversityState>(_modelFields, "biodiversity");

            // Initial values, should eventually also come from parameter file
            // Carefull, the initial value is for the year 50!
            BioDiversityState.nospecies[Timestep.FromSimulationYear(0)] = BioDiversityState.nospecbase;
        }

        public void RunBioDiversityComponent(Clock clock)
        {
            // Set up biodiv component
            var bc = new Fund.Components.BioDiversity.BioDiversityComponent();

            // Run biodiv component
            bc.Run(clock, BioDiversityState);
        }

        public void InitImpactBioDiversityState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactbiodiversity", "nospecbase", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactbiodiversity", "bioshare", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactbiodiversity", "spbm", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactbiodiversity", "valbase", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactbiodiversity", "dbsta", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactbiodiversity", "valinc", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactbiodiversity", "temp", "climateregional", "temp");
            _modelFields.ConnectParameterToVariable("impactbiodiversity", "nospecies", "biodiversity", "nospecies");

            // Parameters coming from different components, explicitly coupled
            _modelFields.ConnectParameterToVariable("impactbiodiversity", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactbiodiversity", "population", "socioeconomic", "population");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactBioDiversity.IImpactBioDiversityState));
            ImpactBioDiversityState = ss.ConnectToState<Fund.Components.ImpactBioDiversity.IImpactBioDiversityState>(_modelFields, "impactbiodiversity");

        }

        public void RunImpactBioDiversityComponent(Clock clock)
        {
            // Set up ImpactBioDiversity component
            var lImpactBioDiversityComponent = new Fund.Components.ImpactBioDiversity.ImpactBioDiversityComponent();

            // Run ImpactBioDiversity component
            lImpactBioDiversityComponent.Run(clock, ImpactBioDiversityState);
        }

        public void InitVariables(System.Type stateinterface, string componentName)
        {
            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(stateinterface);
            ss.CreateVariables(_modelFields, componentName);
        }

        public void InitFundOld2ClimateState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<Timestep>("fundold2climate", "emissionPeriod", Timestep.FromYear(2005));
            _modelFields.AddNonDimensionalVariable<MarginalGas>("fundold2climate", "marginalGas", MarginalGas.C);

            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2Climate));
            FundOld2ClimateState = ss.ConnectToState<Fund.IFundOld2Climate>(_modelFields, "fundold2climate");
        }

        public void RunFundOld2ClimateState(Clock clock)
        {
            var c = new Fund.FundOld2ClimateComponent();

            c.Run(clock, FundOld2ClimateState);
        }

        public void InitGlobalWelfareState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("globalwelfare", "prtp", 0.03);
            _modelFields.AddNonDimensionalVariable<double>("globalwelfare", "elasticityofmarginalutility", 1.0);
            _modelFields.AddNonDimensionalVariable<double>("globalwelfare", "utilitycalibrationadditive", 0.0);
            _modelFields.AddNonDimensionalVariable<double>("globalwelfare", "utilitycalibrationmultiplicative", 1.0);
            _modelFields.AddNonDimensionalVariable<Timestep>("globalwelfare", "starttimestep", Timestep.FromYear(2000));


            // Parameters coming from different components, need to be coupled
            _modelFields.ConnectParameterToVariable("globalwelfare", "consumption", "fundold2globalwelfare", "totalconsumption");
            _modelFields.ConnectParameterToVariable("globalwelfare", "population", "fundold2globalwelfare", "totalpopulation");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IGlobalWelfareState));
            GlobalWelfareState = ss.ConnectToState<Fund.Components.Welfare.IGlobalWelfareState>(_modelFields, "globalwelfare");

            // Initial values, should eventually also come from parameter file
            var t0 = Timestep.FromSimulationYear(0);
            //WelfareState.welfare[t0]=0;
        }

        public void InitFundOld2GlobalWelfare(ParameterValues parameters)
        {
            // Parameters coming from different components, need to be coupled
            _modelFields.ConnectParameterToVariable("fundold2globalwelfare", "consumption", "socioeconomic", "consumption");
            _modelFields.ConnectParameterToVariable("fundold2globalwelfare", "population", "socioeconomic", "population");


            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2Welfare));
            FundOld2GlobalWelfareState = ss.ConnectToState<Fund.ISocioEconomic2Welfare>(_modelFields, "fundold2globalwelfare");
        }

        public void RunFundOld2GlobalWelfareState(Clock clock)
        {
            var c = new Fund.SocioEconomic2Welfare();

            c.Run(clock, FundOld2GlobalWelfareState);
        }

        public void RunGlobalWelfareComponent(Clock clock)
        {
            // Set up SocioEconomic component
            var lComponent = new Fund.Components.Welfare.GlobalWelfareComponent();

            // Run SocioEconomic component
            lComponent.Run(clock, GlobalWelfareState);
        }

        public void InitRegionalWelfareState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("regionalwelfare", "prtp", 0.03);
            _modelFields.AddNonDimensionalVariable<double>("regionalwelfare", "elasticityofmarginalutility", 1.0);
            _modelFields.AddNonDimensionalVariable<double>("regionalwelfare", "utilitycalibrationadditive", 0.0);
            _modelFields.AddNonDimensionalVariable<double>("regionalwelfare", "utilitycalibrationmultiplicative", 1.0);
            _modelFields.AddNonDimensionalVariable<Timestep>("regionalwelfare", "starttimestep", Timestep.FromYear(2000));
            _modelFields.Add2DimensionalParameter<Timestep, Fund.CommonDimensions.Region, Double>("regionalwelfare", "welfareweight",
                (Timestep t, Fund.CommonDimensions.Region r) => 1.0);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("regionalwelfare", "consumption", "socioeconomic", "consumption");

            // Parameters coming from different components, need to be coupled
            _modelFields.ConnectParameterToVariable("regionalwelfare", "population", "socioeconomic", "populationin1");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IRegionalWelfareState));
            RegionalWelfareState = ss.ConnectToState<Fund.Components.Welfare.IRegionalWelfareState>(_modelFields, "regionalwelfare");

        }

        public void RunRegionalWelfareComponent(Clock clock)
        {
            // Set up SocioEconomic component
            var lComponent = new Fund.Components.Welfare.RegionalWelfareComponent();

            // Run SocioEconomic component
            lComponent.Run(clock, RegionalWelfareState);
        }

        public void InitScenarioUncertaintyState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<Timestep>("scenariouncertainty", "timeofuncertaintystart", parameters);

            // Parameters coming from file
            _modelFields.LoadTwoDimensionalVariableFromParameters<Timestep, Fund.CommonDimensions.Region, double>("scenariouncertainty", "inputypcgrowth", parameters, "ypcgrowth");
            _modelFields.LoadTwoDimensionalVariableFromParameters<Timestep, Fund.CommonDimensions.Region, double>("scenariouncertainty", "inputpgrowth", parameters, "pgrowth");
            _modelFields.LoadTwoDimensionalVariableFromParameters<Timestep, Fund.CommonDimensions.Region, double>("scenariouncertainty", "inputaeei", parameters, "AEEI");
            _modelFields.LoadTwoDimensionalVariableFromParameters<Timestep, Fund.CommonDimensions.Region, double>("scenariouncertainty", "inputacei", parameters, "ACEI");
            _modelFields.LoadTwoDimensionalVariableFromParameters<Timestep, Fund.CommonDimensions.Region, double>("scenariouncertainty", "inputForestEmm", parameters, "ForestEmm");
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("scenariouncertainty", "ecgradd", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("scenariouncertainty", "pgadd", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("scenariouncertainty", "aeeiadd", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("scenariouncertainty", "aceiadd", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("scenariouncertainty", "foremadd", parameters);

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState));
            ScenarioUncertaintyState = ss.ConnectToState<Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState>(_modelFields, "scenariouncertainty");
        }

        public void RunScenarioUncertaintyComponent(Clock clock)
        {
            var c = new Fund.Components.ScenarioUncertainty.ScenarioUncertaintyComponent();

            c.Run(clock, ScenarioUncertaintyState);
        }

        public void InitImpactHeatingState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactheating", "heel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactheating", "henl", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactheating", "hebm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactheating", "gdp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactheating", "pop90", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactheating", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactheating", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactheating", "temp", "climateregional", "temp");

            // Parameters coming from different components, explicitly coupled
            _modelFields.ConnectParameterToVariable("impactheating", "cumaeei", "emissions", "cumaeei");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactHeating.IImpactHeatingState));
            ImpactHeatingState = ss.ConnectToState<Fund.Components.ImpactHeating.IImpactHeatingState>(_modelFields, "impactheating");
        }

        public void RunImpactHeatingComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactHeating.ImpactHeatingComponent();

            // Run ImpactBioDiversity component
            c.Run(clock, ImpactHeatingState);
        }

        public void InitImpactCoolingState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactcooling", "ceel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactcooling", "cenl", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcooling", "cebm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcooling", "gdp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcooling", "pop90", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactcooling", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactcooling", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactcooling", "temp", "climateregional", "temp");

            // Parameters coming from different components, explicitly coupled
            _modelFields.ConnectParameterToVariable("impactcooling", "cumaeei", "emissions", "cumaeei");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactCooling.IImpactCoolingState));
            ImpactCoolingState = ss.ConnectToState<Fund.Components.ImpactCooling.IImpactCoolingState>(_modelFields, "impactcooling");
        }

        public void RunImpactCoolingComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactCooling.ImpactCoolingComponent();

            // Run ImpactBioDiversity component
            c.Run(clock, ImpactCoolingState);
        }

        public void InitImpactAgricultureState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactagriculture", "gdp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactagriculture", "pop90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactagriculture", "agrish0", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactagriculture", "agel", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactagriculture", "agrbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactagriculture", "agtime", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactagriculture", "agnl", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactagriculture", "aglparl", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactagriculture", "aglparq", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactagriculture", "agcbm", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactagriculture", "co2pre", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactagriculture", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactagriculture", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactagriculture", "temp", "climateregional", "temp");
            _modelFields.ConnectParameterToVariable("impactagriculture", "acco2", "climate", "acco2");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAgriculture.IImpactAgricultureState));
            ImpactAgricultureState = ss.ConnectToState<Fund.Components.ImpactAgriculture.IImpactAgricultureState>(_modelFields, "impactagriculture");

            // Initial values, should eventually also come from parameter file
            var t0 = Timestep.FromSimulationYear(0);
            foreach (var r in Dimension<Region>.Values)
            {
                ImpactAgricultureState.agrate[t0, r] = ImpactAgricultureState.agrbm[r] * Math.Pow(0.005 / LegacyConstants.DBsT, ImpactAgricultureState.agnl) * ImpactAgricultureState.agtime[r];
            }
        }

        public void RunImpactAgricultureComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactAgriculture.ImpactAgricultureComponent();

            // Run ImpactBioDiversity component
            c.Run(clock, ImpactAgricultureState);
        }

        public void InitImpactAggregationState(ParameterValues parameters)
        {
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffwater", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffforests", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffheating", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffcooling", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffagcost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffdrycost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffprotcost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffentercost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffhurrdam", false);
            // tropical storms switched off while in development
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffextratropicalstormsdam", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffspecies", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffdeadcost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffmorbcost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffwetcost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffleavecost", false);

            _modelFields.ConnectParameterToVariable("impactaggregation", "income", "socioeconomic", "income");

            _modelFields.ConnectParameterToVariable("impactaggregation", "heating", "impactheating", "heating");
            _modelFields.ConnectParameterToVariable("impactaggregation", "cooling", "impactcooling", "cooling");
            _modelFields.ConnectParameterToVariable("impactaggregation", "agcost", "impactagriculture", "agcost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "species", "impactbiodiversity", "species");
            _modelFields.ConnectParameterToVariable("impactaggregation", "water", "impactwaterresources", "water");
            _modelFields.ConnectParameterToVariable("impactaggregation", "hurrdam", "impacttropicalstorms", "hurrdam");
            _modelFields.ConnectParameterToVariable("impactaggregation", "extratropicalstormsdam", "impactextratropicalstorms", "extratropicalstormsdam");

            _modelFields.ConnectParameterToVariable("impactaggregation", "forests", "impactforests", "forests");
            _modelFields.ConnectParameterToVariable("impactaggregation", "drycost", "impactsealevelrise", "drycost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "protcost", "impactsealevelrise", "protcost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "entercost", "impactsealevelrise", "entercost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "deadcost", "impactdeathmorbidity", "deadcost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "morbcost", "impactdeathmorbidity", "morbcost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "wetcost", "impactsealevelrise", "wetcost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "leavecost", "impactsealevelrise", "leavecost");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAggregation.IImpactAggregationState));
            ImpactAggregationState = ss.ConnectToState<Fund.Components.ImpactAggregation.IImpactAggregationState>(_modelFields, "impactaggregation");

            // Initial values, should eventually also come from parameter file
            foreach (var r in Dimension<Region>.Values)
            {
                ImpactAggregationState.eloss[Timestep.FromSimulationYear(0), r] = 0.0;
                ImpactAggregationState.sloss[Timestep.FromSimulationYear(0), r] = 0.0;
            }
        }

        public void RunImpactAggregationComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactAggregation.ImpactAggregationComponent();

            c.Run(clock, ImpactAggregationState);
        }

        public void InitGlobalHotellingTaxationState(ParameterValues parameters)
        {
            _modelFields.AddNonDimensionalVariable<double>("globalhotellingtaxation", "prtp", 0.03);
            _modelFields.AddNonDimensionalVariable<double>("globalhotellingtaxation", "elasticityofmarginalutility", 1.0);
            _modelFields.AddNonDimensionalVariable<double>("globalhotellingtaxation", "basetax", 0.0);
            _modelFields.AddNonDimensionalVariable<Timestep>("globalhotellingtaxation", "baseyear", Timestep.FromSimulationYear(50));

            _modelFields.ConnectParameterToVariable("globalhotellingtaxation", "consumption", "socioeconomic", "globalconsumption");
            _modelFields.ConnectParameterToVariable("globalhotellingtaxation", "population", "socioeconomic", "globalpopulation");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState));
            GlobalHotellingTaxationState = ss.ConnectToState<Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState>(_modelFields, "globalhotellingtaxation");

        }

        public void RunGlobalHotellingTaxationComponent(Clock clock)
        {
            var c = new Fund.Components.HotellingTaxation.GlobalHotellingTaxationComponent();

            c.Run(clock, GlobalHotellingTaxationState);

        }

        public void ReCreateStateVariables()
        {
            // Create proxy
            var s1 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.SocioEconomic.ISocioEconomicState));
            SocioEconomicState = s1.ConnectToState<Fund.Components.SocioEconomic.ISocioEconomicState>(_modelFields, "socioeconomic");

            var s2 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IGlobalWelfareState));
            GlobalWelfareState = s2.ConnectToState<Fund.Components.Welfare.IGlobalWelfareState>(_modelFields, "globalwelfare");

            var s3 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IRegionalWelfareState));
            RegionalWelfareState = s3.ConnectToState<Fund.Components.Welfare.IRegionalWelfareState>(_modelFields, "regionalwelfare");

            var s4 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAggregation.IImpactAggregationState));
            ImpactAggregationState = s4.ConnectToState<Fund.Components.ImpactAggregation.IImpactAggregationState>(_modelFields, "impactaggregation");

            var s5 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Climate.IClimateState));
            ClimateState = s5.ConnectToState<Fund.Components.Climate.IClimateState>(_modelFields, "climate");

            var s6 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Ocean.IOceanState));
            OceanState = s6.ConnectToState<Fund.Components.Ocean.IOceanState>(_modelFields, "ocean");

            var s7 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.BioDiversity.IBioDiversityState));
            BioDiversityState = s7.ConnectToState<Fund.Components.BioDiversity.IBioDiversityState>(_modelFields, "biodiversity");

            var s8 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactBioDiversity.IImpactBioDiversityState));
            ImpactBioDiversityState = s8.ConnectToState<Fund.Components.ImpactBioDiversity.IImpactBioDiversityState>(_modelFields, "impactbiodiversity");

            var s12 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2Climate));
            FundOld2ClimateState = s12.ConnectToState<Fund.IFundOld2Climate>(_modelFields, "fundold2climate");

            var s13 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2Welfare));
            FundOld2GlobalWelfareState = s13.ConnectToState<Fund.ISocioEconomic2Welfare>(_modelFields, "fundold2globalwelfare");

            var s15 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState));
            ScenarioUncertaintyState = s15.ConnectToState<Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState>(_modelFields, "scenariouncertainty");

            var s16 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactHeating.IImpactHeatingState));
            ImpactHeatingState = s16.ConnectToState<Fund.Components.ImpactHeating.IImpactHeatingState>(_modelFields, "impactheating");

            var s17 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactCooling.IImpactCoolingState));
            ImpactCoolingState = s17.ConnectToState<Fund.Components.ImpactCooling.IImpactCoolingState>(_modelFields, "impactcooling");

            var s19 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAgriculture.IImpactAgricultureState));
            ImpactAgricultureState = s19.ConnectToState<Fund.Components.ImpactAgriculture.IImpactAgricultureState>(_modelFields, "impactagriculture");

            var s21 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState));
            GlobalHotellingTaxationState = s21.ConnectToState<Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState>(_modelFields, "globalhotellingtaxation");

            var s22 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactWaterResources.IImpactWaterResourcesState));
            ImpactWaterResourcesState = s22.ConnectToState<Fund.Components.ImpactWaterResources.IImpactWaterResourcesState>(_modelFields, "impactwaterresources");

            var s23 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState));
            ImpactDiarrhoeaState = s23.ConnectToState<Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState>(_modelFields, "impactdiarrhoea");

            var s24 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ClimateRegional.IClimateRegionalState));
            ClimateRegionalState = s24.ConnectToState<Fund.Components.ClimateRegional.IClimateRegionalState>(_modelFields, "climateregional");

            var s25 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState));
            ImpactTropicalStormsState = s25.ConnectToState<Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState>(_modelFields, "impacttropicalstorms");

            var s26 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState));
            ImpactExtratropicalStormsState = s26.ConnectToState<Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState>(_modelFields, "impactextratropicalstorms");

            var s27 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState));
            ImpactSeaLevelRiseState = s27.ConnectToState<Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState>(_modelFields, "impactsealevelrise");

            var s28 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Geography.IGeographyState));
            GeographyState = s28.ConnectToState<Fund.Components.Geography.IGeographyState>(_modelFields, "geography");

            var s29 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Emissions.IEmissionsState));
            EmissionsState = s29.ConnectToState<Fund.Components.Emissions.IEmissionsState>(_modelFields, "emissions");

            var s30 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState));
            TerrestrialBiosphereState = s30.ConnectToState<Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState>(_modelFields, "terrestrialbiosphere");

            var s31 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactForests.IImpactForestsState));
            ImpactForestsState = s31.ConnectToState<Fund.Components.ImpactForests.IImpactForestsState>(_modelFields, "impactforests");

            var s32 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState));
            ImpactVectorBorneDiseases = s32.ConnectToState<Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState>(_modelFields, "impactvectorbornediseases");

            var s33 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState));
            ImpactCardiovascularRespiratoryState = s33.ConnectToState<Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState>(_modelFields, "impactcardiovascularrespiratory");

            var s34 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState));
            ImpactDeathMorbidityState = s34.ConnectToState<Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState>(_modelFields, "impactdeathmorbidity");

        }

        public void InitImpactWaterResourcesState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactwaterresources", "watechrate", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactwaterresources", "wrel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactwaterresources", "wrnl", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactwaterresources", "wrpl", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactwaterresources", "wrbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactwaterresources", "gdp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactwaterresources", "pop90", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactwaterresources", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactwaterresources", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactwaterresources", "temp", "climateregional", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactWaterResources.IImpactWaterResourcesState));
            ImpactWaterResourcesState = ss.ConnectToState<Fund.Components.ImpactWaterResources.IImpactWaterResourcesState>(_modelFields, "impactwaterresources");
        }

        public void RunImpactWaterResourcesComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactWaterResources.ImpactWaterResourcesComponent();

            c.Run(clock, ImpactWaterResourcesState);
        }

        public void InitImpactDiarrhoeaState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactdiarrhoea", "diamortel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactdiarrhoea", "diamortnl", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactdiarrhoea", "diayldel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactdiarrhoea", "diayldnl", parameters);

            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "diamort", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "diayld", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "gdp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "pop90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "temp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "bregtmp", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactdiarrhoea", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactdiarrhoea", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactdiarrhoea", "regtmp", "climateregional", "regtmp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState));
            ImpactDiarrhoeaState = ss.ConnectToState<Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState>(_modelFields, "impactdiarrhoea");
        }

        public void RunImpactDiarrhoeaComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactDiarrhoea.ImpactDiarrhoeaComponent();

            c.Run(clock, ImpactDiarrhoeaState);
        }

        public void InitClimateRegionalState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("climateregional", "bregtmp", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("climateregional", "bregstmp", parameters);
            _modelFields.LoadTwoDimensionalVariableFromParameters<Timestep, Fund.CommonDimensions.Region, double>("climateregional", "scentemp", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("climateregional", "inputtemp", "climate", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ClimateRegional.IClimateRegionalState));
            ClimateRegionalState = ss.ConnectToState<Fund.Components.ClimateRegional.IClimateRegionalState>(_modelFields, "climateregional");
        }

        public void RunClimateRegionalComponent(Clock clock)
        {
            var c = new Fund.Components.ClimateRegional.ClimateRegionalComponent();

            c.Run(clock, ClimateRegionalState);
        }

        public void InitImpactTropicalStormsState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impacttropicalstorms", "hurrdamel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impacttropicalstorms", "hurrnl", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impacttropicalstorms", "hurrpar", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impacttropicalstorms", "hurrdeadel", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impacttropicalstorms", "hurrbasedam", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impacttropicalstorms", "hurrbasedead", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impacttropicalstorms", "gdp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impacttropicalstorms", "pop90", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impacttropicalstorms", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impacttropicalstorms", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impacttropicalstorms", "regstmp", "climateregional", "regstmp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState));
            ImpactTropicalStormsState = ss.ConnectToState<Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState>(_modelFields, "impacttropicalstorms");
        }

        public void RunImpactTropicalStormsComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactTropicalStorms.ImpactTropicalStormsComponent();

            c.Run(clock, ImpactTropicalStormsState);
        }

        public void InitImpactExtratropicalStormsState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactextratropicalstorms", "co2pre", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactextratropicalstorms", "extratropicalstormsdamel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactextratropicalstorms", "extratropicalstormsnl", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactextratropicalstorms", "extratropicalstormspar", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactextratropicalstorms", "extratropicalstormsdeadel", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactextratropicalstorms", "extratropicalstormsbasedam", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactextratropicalstorms", "extratropicalstormsbasedead", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactextratropicalstorms", "gdp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactextratropicalstorms", "pop90", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactextratropicalstorms", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactextratropicalstorms", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactextratropicalstorms", "acco2", "climate", "acco2");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState));
            ImpactExtratropicalStormsState = ss.ConnectToState<Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState>(_modelFields, "impactextratropicalstorms");
        }

        public void RunImpactExtratropicalStormsComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactExtratropicalStorms.ImpactExtratropicalStormsComponent();

            c.Run(clock, ImpactExtratropicalStormsState);
        }

        public void InitImpactSeaLevelRiseState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactsealevelrise", "incdens", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactsealevelrise", "emcst", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactsealevelrise", "immcst", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactsealevelrise", "dvydl", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactsealevelrise", "wvel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactsealevelrise", "wvbm", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactsealevelrise", "slrwvpopdens0", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactsealevelrise", "wvpdl", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactsealevelrise", "wvsl", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactsealevelrise", "dvbm", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactsealevelrise", "slrwvypc0", parameters);

            // Parameters coming from file
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactsealevelrise", "pc", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactsealevelrise", "slrprtp", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactsealevelrise", "wmbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactsealevelrise", "dlbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactsealevelrise", "wlbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactsealevelrise", "coastpd", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactsealevelrise", "wetmax", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactsealevelrise", "wetland90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactsealevelrise", "drylandlossparam", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactsealevelrise", "maxlandloss", parameters);

            _modelFields.LoadTwoDimensionalVariableFromParameters<Fund.CommonDimensions.Region, Fund.CommonDimensions.Region, double>("impactsealevelrise", "migrate", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactsealevelrise", "sea", "ocean", "sea");
            _modelFields.ConnectParameterToVariable("impactsealevelrise", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactsealevelrise", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactsealevelrise", "area", "geography", "area");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState));
            ImpactSeaLevelRiseState = ss.ConnectToState<Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState>(_modelFields, "impactsealevelrise");

        }

        public void RunImpactSeaLevelRiseComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactSeaLevelRise.ImpactSeaLevelRiseComponent();

            c.Run(clock, ImpactSeaLevelRiseState);
        }

        public void InitGeographyState(ParameterValues parameters)
        {
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("geography", "area0", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("geography", "landloss", "impactsealevelrise", "landloss");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Geography.IGeographyState));
            GeographyState = ss.ConnectToState<Fund.Components.Geography.IGeographyState>(_modelFields, "geography");
        }

        public void RunGeographyComponent(Clock clock)
        {
            var c = new Fund.Components.Geography.GeographyComponent();

            c.Run(clock, GeographyState);
        }

        public void InitEmissionsState(ParameterValues parameters)
        {
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "so2pop", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "so2inc", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "so2carb", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "sf6gdp", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "sf6ypc", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "knowpar", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "knowgpar", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "gwpch4", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "gwpn2o", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "TaxConstant", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "TaxEmInt", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "TaxThreshold", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "TaxDepreciation", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("emissions", "MaxCostFall", parameters);

            _modelFields.ConnectParameterToVariable("emissions", "aeei", "scenariouncertainty", "aeei");
            _modelFields.ConnectParameterToVariable("emissions", "acei", "scenariouncertainty", "acei");
            _modelFields.ConnectParameterToVariable("emissions", "forestemm", "scenariouncertainty", "ForestEmm");

            _modelFields.ConnectParameterToVariable("emissions", "currtax", "globalhotellingtaxation", "currtax");
            _modelFields.ConnectParameterToVariable("emissions", "currtaxch4", "globalhotellingtaxation", "currtaxch4");
            _modelFields.ConnectParameterToVariable("emissions", "currtaxn2o", "globalhotellingtaxation", "currtaxn2o");

            _modelFields.ConnectParameterToVariable("emissions", "pgrowth", "scenariouncertainty", "pgrowth");
            _modelFields.ConnectParameterToVariable("emissions", "ypcgrowth", "scenariouncertainty", "ypcgrowth");
            _modelFields.ConnectParameterToVariable("emissions", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("emissions", "population", "socioeconomic", "population");


            // Parameters coming from file
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("emissions", "taxmp", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("emissions", "gdp0", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("emissions", "sf60", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("emissions", "GDP90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("emissions", "pop90", parameters);
            _modelFields.LoadTwoDimensionalVariableFromParameters<Timestep, Fund.CommonDimensions.Region, double>("emissions", "ch4em", parameters);
            _modelFields.LoadTwoDimensionalVariableFromParameters<Timestep, Fund.CommonDimensions.Region, double>("emissions", "n2oem", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("emissions", "ch4par1", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("emissions", "ch4par2", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("emissions", "n2opar1", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("emissions", "n2opar2", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("emissions", "emissint0", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("emissions", "so20", parameters);

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Emissions.IEmissionsState));
            EmissionsState = ss.ConnectToState<Fund.Components.Emissions.IEmissionsState>(_modelFields, "emissions");


        }

        public void RunEmissionsComponent(Clock clock)
        {
            // Set up SocioEconomic component
            var c = new Fund.Components.Emissions.EmissionsComponent();

            // Run SocioEconomic component
            c.Run(clock, EmissionsState);
        }

        public void InitTerrestrialBiosphereState(ParameterValues parameters)
        {
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("terrestrialbiosphere", "TerrCO2Sens", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("terrestrialbiosphere", "TerrCO2Stock0", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("terrestrialbiosphere", "temp", "climate", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState));
            TerrestrialBiosphereState = ss.ConnectToState<Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState>(_modelFields, "terrestrialbiosphere");
        }

        public void RunTerrestrialBiosphereComponent(Clock clock)
        {
            // Set up SocioEconomic component
            var c = new Fund.Components.TerrestrialBiosphere.TerrestrialBiosphereComponent();

            // Run SocioEconomic component
            c.Run(clock, TerrestrialBiosphereState);
        }

        public void InitImpactForestsState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactforests", "forel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactforests", "fornl", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactforests", "co2pre", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactforests", "forco2", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactforests", "forbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactforests", "gdp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactforests", "pop90", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactforests", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactforests", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactforests", "temp", "climateregional", "temp");
            _modelFields.ConnectParameterToVariable("impactforests", "acco2", "climate", "acco2");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactForests.IImpactForestsState));
            ImpactForestsState = ss.ConnectToState<Fund.Components.ImpactForests.IImpactForestsState>(_modelFields, "impactforests");
        }

        public void RunImpactForestsComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactForests.ImpactForests();

            c.Run(clock, ImpactForestsState);
        }

        public void InitImpactVectorBorneDiseasesState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactvectorbornediseases", "dfnl", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactvectorbornediseases", "vbel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactvectorbornediseases", "smnl", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactvectorbornediseases", "malnl", parameters);

            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "dfbs", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "dfch", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "smbs", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "smch", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "malbs", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "malch", parameters);

            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "gdp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "pop90", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactvectorbornediseases", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactvectorbornediseases", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactvectorbornediseases", "temp", "climateregional", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState));
            ImpactVectorBorneDiseases = ss.ConnectToState<Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState>(_modelFields, "impactvectorbornediseases");
        }

        public void RunImpactVectorBorneDiseasesComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactVectorBorneDiseases.ImpactVectorBorneDiseasesComponent();

            c.Run(clock, ImpactVectorBorneDiseases);
        }

        public void InitImpactCardiovascularRespiratoryState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactcardiovascularrespiratory", "cvlin", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactcardiovascularrespiratory", "rlin", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactcardiovascularrespiratory", "maxcardvasc", parameters);

            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "cardvasc90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "plus90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "resp90", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "chplbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "chmlbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "chpqbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "chmqbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "rlbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "rqbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "ccplbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "ccmlbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "ccpqbm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "ccmqbm", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactcardiovascularrespiratory", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactcardiovascularrespiratory", "plus", "socioeconomic", "plus");
            _modelFields.ConnectParameterToVariable("impactcardiovascularrespiratory", "urbpop", "socioeconomic", "urbpop");
            _modelFields.ConnectParameterToVariable("impactcardiovascularrespiratory", "temp", "climateregional", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState));
            ImpactCardiovascularRespiratoryState = ss.ConnectToState<Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState>(_modelFields, "impactcardiovascularrespiratory");
        }

        public void RunImpactCardiovascularRespiratoryComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactCardiovascularRespiratory.ImpactCardiovascularRespiratoryComponent();

            c.Run(clock, ImpactCardiovascularRespiratoryState);
        }

        public void InitImpactDeathMorbidityState(ParameterValues parameters)
        {
            // Constant parameters
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactdeathmorbidity", "vslbm", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactdeathmorbidity", "vslel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactdeathmorbidity", "vmorbbm", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactdeathmorbidity", "vmorbel", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactdeathmorbidity", "vslypc0", parameters);
            _modelFields.LoadNonDimensionalVariableFromParameters<double>("impactdeathmorbidity", "vmorbypc0", parameters);


            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2ld", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2ls", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2lm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2lc", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2lr", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2dd", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2ds", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2dm", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2dc", parameters);
            _modelFields.LoadOneDimensionalVariableFromParameters<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2dr", parameters);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "dengue", "impactvectorbornediseases", "dengue");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "schisto", "impactvectorbornediseases", "schisto");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "malaria", "impactvectorbornediseases", "malaria");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "cardheat", "impactcardiovascularrespiratory", "cardheat");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "cardcold", "impactcardiovascularrespiratory", "cardcold");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "resp", "impactcardiovascularrespiratory", "resp");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "diadead", "impactdiarrhoea", "diadead");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "hurrdead", "impacttropicalstorms", "hurrdead");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "extratropicalstormsdead", "impactextratropicalstorms", "extratropicalstormsdead");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "diasick", "impactdiarrhoea", "diasick");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState));
            ImpactDeathMorbidityState = ss.ConnectToState<Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState>(_modelFields, "impactdeathmorbidity");
        }

        public void RunImpactDeathMorbidityComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactDeathMorbidity.ImpactDeathMorbidityComponent();

            c.Run(clock, ImpactDeathMorbidityState);
        }

    }
}