﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;

namespace Fund
{
    public static class LegacyFund28Math
    {

        public static double Sqr(double x)
        {
            return Math.Pow(x, 2);
        }

        public static double Exp(double x)
        {
            return Math.Exp(x);
        }

        public static double sqrt(double x)
        {
            return Math.Sqrt(x);
        }

        public static double ln(double x)
        {
            return Math.Log(x);
        }

        public static double Log(double x)
        //(* returns ln(x) *)
        {
            if ((x > 1.0E-38) && (x < 1.0E38))
                return ln(x);
            else if (x < 1.0E-38)
                return ln(1.0E-38);
            else
                return ln(1.0E38);
        }

        public static double Expo(double x)
        //(* returns exp(x) *)
        {
            if ((x < 88) && (x > -88))
                return Exp(x);
            else if (x < -88)
                return Exp(-88);
            else
                return Exp(88);
        }

        public static double Pow(double x, double y)
        //(* returns x^y *)
        {
            return Expo(y * Log(x));
        }

        public static void GenNormal(double mu, double sigma, out double a, out double b, Random rand)
        {
            double u, v, w;


            u = 2 * rand.NextDouble() - 1;
            v = 2 * rand.NextDouble() - 1;
            w = Sqr(u) + Sqr(v);
            if (w > 1)
                GenNormal(mu, sigma, out a, out b, rand);
            else
            {
                w = sqrt(-2 * Log(w) / w);
                a = mu + sigma * w * u;
                b = mu + sigma * w * v;
            }
        }

        public static void GenNormGTZ(double mu, double sigma, out double a, out double b, Random rand)
        {
            GenNormal(mu, sigma, out a, out b, rand);
            while ((a < 0) || (b < 0))
                GenNormal(mu, sigma, out a, out b, rand);
        }

        public static void GenNormKnot(double mu, double sigma, double l, double u, out double a, out double b, Random rand)
        {
            GenNormal(mu, sigma, out a, out b, rand);
            while ((a < l) || (b < l) || (a > u) || (b > u))
                GenNormal(mu, sigma, out a, out b, rand);
        }

        public static void GenLogNormal(double mu, double sigma, out double a, out double b, Random rand)
        {
            GenNormal(mu, sigma, out a, out b, rand);
            a = Expo(a);
            b = Expo(b);
        }

        public static void GenGumbel(double dzeta, double theta, out double a, Random rand)
        {
            a = dzeta - theta * Log(-Log(rand.NextDouble()));
        }

        public static void GenExponential(double lambda, out double a, Random rand)
        {
            a = -lambda * Log(rand.NextDouble());
        }

        public static void GamLTOne(double alpha, out double a, Random rand)
        {
            double e, b, p, y;

            e = Expo(1);
            b = (e + alpha) / e;
            p = b * rand.NextDouble();
            if (p < 1)
            {
                y = Expo(Log(p) / alpha);
                if (Expo(-y) >= rand.NextDouble())
                    a = y;
                else
                    GamLTOne(alpha, out a, rand);
            }
            else
            {
                y = -Log((b - p) / alpha);
                if (Expo((alpha - 1) * Log(y)) >= rand.NextDouble())
                    a = y;
                else
                    GamLTOne(alpha, out a, rand);
            }
        }

        public static void GamGTOne(double alpha, out double a, Random rand)
        {
            double ha, b, q, th, d, u1, u2, v, y, z, w;


            ha = 1 / sqrt(2 * alpha - 1);
            b = alpha - Log(4);
            q = alpha + 1 / ha;
            th = 4.5;
            d = 1 + Log(th);
            u1 = rand.NextDouble();
            u2 = rand.NextDouble();
            v = ha * Log(u1 / (1 - u1));
            y = alpha * Expo(v);
            z = u1 * u1 * u2;
            w = b + q * v - y;
            if ((w + d - th * z) >= 0)
                a = y;
            else if (w >= Log(z))
                a = y;
            else
                GamGTOne(alpha, out a, rand);
        }

        public static void GenGamma(double alpha, double beta, out double a, Random rand)
        {
            if (alpha < 1)
            {
                GamLTOne(alpha, out a, rand);
                a = beta * a;
            }
            else if (alpha == 1)
            {
                GenExponential(1, out a, rand);
                a = beta * a;
            }
            else
            {
                GamGTOne(alpha, out a, rand);
                a = beta * a;
            }
        }

        public static void GenPareto(double alpha, double kappa, out double a, Random rand)
        {
            a = kappa * Expo(-Log(1 - rand.NextDouble()) / alpha);
        }

        public static void GenTriang(double a, double b, double c, out double x, Random rand)
        {
            double d, u, h;


            d = (c - a) / (b - a);
            u = rand.NextDouble();
            if (u <= d)
                h = sqrt(d * u);
            else
                h = 1 - sqrt((1 - d) * (1 - u));
            x = a + (b - a) * h;
        }
    }
}