﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Esmf;
using Fund.CommonDimensions;


namespace Fund.Components.Welfare
{

    public interface IGlobalWelfareState
    {
        Double prtp { get; }
        Double elasticityofmarginalutility { get; }
        Double utilitycalibrationadditive { get; }
        Double utilitycalibrationmultiplicative { get; }

        Timestep starttimestep { get; }

        IParameter1Dimensional<Timestep, double> population { get; }
        IParameter1Dimensional<Timestep, double> consumption { get; }

        IVariable1Dimensional<Timestep, double> cummulativewelfare { get; }
        IVariable1Dimensional<Timestep, double> marginalwelfare { get; }

        Double totalwelfare { get; set; }
    }

    public interface IRegionalWelfareState
    {
        Double prtp { get; }
        Double elasticityofmarginalutility { get; }
        Double utilitycalibrationadditive { get; }
        Double utilitycalibrationmultiplicative { get; }

        Timestep starttimestep { get; }

        IParameter2Dimensional<Timestep, Region, double> population { get; }
        IParameter2Dimensional<Timestep, Region, double> consumption { get; }
        IParameter2Dimensional<Timestep, Region, double> welfareweight { get; }

        IVariable1Dimensional<Timestep, double> cummulativewelfare { get; }
        IVariable2Dimensional<Timestep, Region, double> marginalwelfare { get; }

        Double totalwelfare { get; set; }
    }

    public class GlobalWelfareComponent
    {
        public void Run(Clock clock, IGlobalWelfareState state)
        {
            var s = state;
            var t = clock.Current;

            if (t >= s.starttimestep)
            {

                var U = Funcifier.Funcify(
                    (double consumption) =>
                    {
                        if (s.elasticityofmarginalutility == 1.0)
                            return s.utilitycalibrationadditive + s.utilitycalibrationmultiplicative * Math.Log(consumption);
                        else
                            return s.utilitycalibrationadditive + s.utilitycalibrationmultiplicative * Math.Pow(consumption, 1.0 - s.elasticityofmarginalutility) / (1.0 - s.elasticityofmarginalutility);
                    }
                    );

                var DF = Funcifier.Funcify(
                    (Timestep year) =>
                    {
                        return Math.Pow(1.0 + s.prtp, -(year.Value - s.starttimestep.Value));
                    }
                    );

                var perCapitaConsumption = s.consumption[t] / s.population[t];

                if (perCapitaConsumption <= 0.0)
                    perCapitaConsumption = 1.0;

                s.cummulativewelfare[t] = s.cummulativewelfare[t - 1] + (U(perCapitaConsumption) * s.population[t] * DF(t));
                s.marginalwelfare[t] = DF(t) * s.utilitycalibrationmultiplicative / Math.Pow(perCapitaConsumption, s.elasticityofmarginalutility);

                if (clock.IsLastTimestep)
                {
                    s.totalwelfare = s.cummulativewelfare[t];
                }
            }
            else
                s.cummulativewelfare[t] = 0;
        }
    }

    public class RegionalWelfareComponent
    {
        public void Run(Clock clock, IRegionalWelfareState state)
        {
            var s = state;
            var t = clock.Current;

            if (t >= s.starttimestep)
            {
                var w = s.cummulativewelfare[t - 1];

                var U = Funcifier.Funcify(
                    (double consumption) =>
                    {
                        if (s.elasticityofmarginalutility == 1.0)
                            return s.utilitycalibrationadditive + s.utilitycalibrationmultiplicative * Math.Log(consumption);
                        else
                            return s.utilitycalibrationadditive + s.utilitycalibrationmultiplicative * Math.Pow(consumption, 1.0 - s.elasticityofmarginalutility) / (1.0 - s.elasticityofmarginalutility);

                    }
                    );

                var DF = Funcifier.Funcify(
                    (Timestep year) =>
                    {
                        return Math.Pow(1.0 + s.prtp, -(year.Value - s.starttimestep.Value));
                    }
                  );

                foreach (var r in Dimension<Region>.Values)
                {
                    var perCapitaConsumption = s.consumption[t, r] / s.population[t, r];

                    // This is a lower bound
                    if (perCapitaConsumption <= 0.0)
                        perCapitaConsumption = 1.0;

                    w = w + (s.welfareweight[t, r] * U(perCapitaConsumption) * s.population[t, r] * DF(t));
                    s.marginalwelfare[t, r] = DF(t) * s.welfareweight[t, r] * s.utilitycalibrationmultiplicative / Math.Pow(perCapitaConsumption, s.elasticityofmarginalutility);
                }
                s.cummulativewelfare[t] = w;

                if (clock.IsLastTimestep)
                    s.totalwelfare = s.cummulativewelfare[t];
            }
            else
                s.cummulativewelfare[t] = 0;
        }
    }

}
